;;;======================================================
;;;   Sample JESS rules
;;;
;;;     Shows early expert system capability in PAM.
;;;     Watches changes in agent properties to detect when
;;;	an incoming product has been received by a given agent
;;;	and triggers on that event by activating another agent.
;;;
;;;   Uses custom additions to Jess command set:
;;;    (editAgent <agent-id> removeProperty <propertyName>)
;;;    (editAgent <agent-id> setProperty <propertyName> <propertyValue>)
;;;    (editAgent <agent-id> createOfType <agentType>)
;;;    (editAgent <agent-id> destroy)
;;;    (editAgent <agent-id> connectOutputTo <other-agent-id>)
;;;    (editAgent <agent-id> disconnectOutputTo <other-agent-id>)
;;;    (editAgent <agent-id> activateASAP)
;;;    (assertAgent <agent-id> [fact-name])
;;;    (assertAgentWithAttribute attrname attrvalue [attrname attrvalue]* )
;;;    (retractAgent <agent-id>)
;;;    (writeToLog log|warn <msg>)
;;;
;;;======================================================

(writeToLog log "start loading JESS rules")

; Initialize the product counter to zero before we do anything
(editAgent "test ProductCounter" setProperty productCount 0)

; Ensure that the test juggler is enabled
(editAgent "test activator" setProperty enabled true)

; The following rule triggers when the following events happen:
;  1) a product is received by the product counter juggler.
;  2) product counter is incremented and the counter property is updated.
;  3) assertAgent is notified of a change in the counter property and
;     creates a fact in JESS of that property change.
;  4) This rule matches the new fact and fires up.

(defrule changeCounter "trigger upon new products"
        ?f <- (pamAgent3 "test ProductCounter" productCount ?count&~0)
  =>
 (printout t "Product has been received " ?count crlf)

 ; immediately activate another agent
 (printout t "Activate juggler -> test activator2" crlf)
 (editAgent "test activator2" activateASAP)

 ; deactivate first agent
 (editAgent "test activator" setProperty enabled false)
 (printout t "Deactivate juggler -> test activator" crlf)

 (printout t "End of test" crlf)
 (printout t crlf "Manually enable 'test activator' Juggler to re-run the test" crlf)
)

(printout t "Expert system is starting up" crlf)

; Register to be notified upon property changes to product counter
(assertAgent "test ProductCounter")

(writeToLog log "end loading JESS rules")
